#include <QGuiApplication>
#include <QList>
#include <QQmlApplicationEngine>
#include <QQmlContext>

class MyEnums : public QObject {
  Q_OBJECT

 public:
  enum class Enum1 { VALUE1, VALUE2 };
  Q_ENUM(Enum1)
};

Q_DECLARE_METATYPE(MyEnums::Enum1)
Q_DECLARE_METATYPE(QList<MyEnums::Enum1>)

class MyClass : public QObject {
  Q_OBJECT
 public:
  Q_INVOKABLE QList<MyEnums::Enum1> test() {
      return QList<MyEnums::Enum1>{MyEnums::Enum1::VALUE1,
                                 MyEnums::Enum1::VALUE2};
  }
};

int main(int argc, char *argv[]) {
  QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);

  QGuiApplication app(argc, argv);

  QQmlApplicationEngine engine;

  qmlRegisterUncreatableType<MyEnums>("MyEnums", 1, 0, "MyEnums",
                                      "Import all enums from MyEnums");
  qRegisterMetaType<QList<MyEnums::Enum1>>("QList<MyEnums::Enum1>");

  MyClass obj;
  engine.rootContext()->setContextProperty("myclass", &obj);

  QVariantList l{QVariant::fromValue(MyEnums::Enum1::VALUE1),
                 QVariant::fromValue(MyEnums::Enum1::VALUE2)};
  engine.rootContext()->setContextProperty("l", QVariant::fromValue(l));

  const QUrl url(QStringLiteral("qrc:/main.qml"));
  QObject::connect(&engine, &QQmlApplicationEngine::objectCreated, &app,
                   [url](QObject *obj, const QUrl &objUrl) {
                     if (!obj && url == objUrl) QCoreApplication::exit(-1);
                   },
                   Qt::QueuedConnection);
  engine.load(url);

  return app.exec();
}

#include "main.moc"
